@@ -4,6 +4,7 @@ import subprocess
 import random
 import paramiko
 import csle_collector.constants.constants as collector_constants
+import csle_ryu.constants.constants as ryu_constants
 import csle_common.constants.constants as constants
 import csle_rest_api.constants.constants as api_constants
 from csle_common.dao.emulation_config.emulation_env_config import EmulationEnvConfig
@@ -80,15 +81,17 @@ class EmulationEnvController:
             ContainerController.stop_docker_stats_thread(execution=exec)
 
     @staticmethod
-    def install_csle_collector_library(emulation_env_config: EmulationEnvConfig):
+    def install_csle_collector_and_ryu_libraries(emulation_env_config: EmulationEnvConfig):
         """
-        Installs the latest csle-collector library on all nodes
+        Installs the latest csle-collector and csle-ryu libraries on all nodes
 
         :return:
         """
         ips = list(map(lambda x: x.get_ips()[0], emulation_env_config.containers_config.containers))
         ips.append(emulation_env_config.kafka_config.container.get_ips()[0])
         ips.append(emulation_env_config.elk_config.container.get_ips()[0])
+        if emulation_env_config.sdn_controller_config is not None:
+            ips.append(emulation_env_config.sdn_controller_config.container.get_ips()[0])
         for ip in ips:
             Logger.__call__().get_logger().info(f"Installing csle-collector version "
                                                 f"{emulation_env_config.csle_collector_version} on node: {ip}")
@@ -99,6 +102,17 @@ class EmulationEnvController:
             o, e, _ = EmulationUtil.execute_ssh_cmd(cmd=cmd, conn=emulation_env_config.get_connection(ip=ip))
             time.sleep(2)
             o, e, _ = EmulationUtil.execute_ssh_cmd(cmd=cmd, conn=emulation_env_config.get_connection(ip=ip))
+            time.sleep(2)
+            Logger.__call__().get_logger().info(f"Installing csle-ryu version "
+                                                f"{emulation_env_config.csle_ryu_version} on node: {ip}")
+            EmulationUtil.connect_admin(emulation_env_config=emulation_env_config, ip=ip)
+            cmd = ryu_constants.INSTALL
+            if emulation_env_config.csle_ryu_version != ryu_constants.LATEST_VERSION:
+                cmd = cmd + f"=={emulation_env_config.csle_ryu_version}"
+            o, e, _ = EmulationUtil.execute_ssh_cmd(cmd=cmd, conn=emulation_env_config.get_connection(ip=ip))
+            time.sleep(2)
+            o, e, _ = EmulationUtil.execute_ssh_cmd(cmd=cmd, conn=emulation_env_config.get_connection(ip=ip))
+
             EmulationUtil.disconnect_admin(emulation_env_config=emulation_env_config)
 
     @staticmethod
@@ -132,7 +146,7 @@ class EmulationEnvController:
 
         current_step += 1
         Logger.__call__().get_logger().info(f"-- Step {current_step}/{steps}: Install csle-collector --")
-        EmulationEnvController.install_csle_collector_library(emulation_env_config=emulation_env_config)
+        EmulationEnvController.install_csle_collector_and_ryu_libraries(emulation_env_config=emulation_env_config)
 
         current_step += 1
         Logger.__call__().get_logger().info(f"-- Step {current_step}/{steps}: Apply kafka config --")
@@ -490,7 +504,7 @@ class EmulationEnvController:
             # Start the SDN controller container
             c = emulation_env_config.sdn_controller_config.container
             container_resources: NodeResourcesConfig = emulation_env_config.sdn_controller_config.resources
-            name = f"{constants.CSLE.NAME}-{c.name}{c.suffix}-level{c.level}-{c.execution_ip_first_octet}"
+            name = c.get_full_name()
             Logger.__call__().get_logger().info(f"Starting container:{name}")
             cmd = f"docker container run -dt --name {name} " \
                   f"--hostname={c.name}{c.suffix} --label dir={path} " \
@@ -854,13 +868,16 @@ class EmulationEnvController:
         return execution_info
 
     @staticmethod
-    def create_kibana_tunnel(execution: EmulationExecution, tunnels_dict: Dict[str, Any], local_port: int) -> None:
+    def create_ssh_tunnel(tunnels_dict: Dict[str, Any], local_port: int,
+                          remote_port: int, remote_ip: str) -> None:
         """
-        Creates an SSH tunnel to forward the Kibana port of a container
+        Creates an SSH tunnel to forward the port of a container
 
         :param execution: the emulation execution
         :param tunnels_dict: a dict with existing tunnels
         :param local_port: the local port to forward
+        :param remote_port: the remote port to forward
+        :param remote_ip: the remote ip to forward
         :return: None
         """
         config = Config.get_current_confg()
@@ -868,17 +885,17 @@ class EmulationEnvController:
             Config.set_config_parameters_from_config_file()
         conn = paramiko.SSHClient()
         conn.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-        conn.connect(execution.emulation_env_config.elk_config.container.get_ips()[0],
+        conn.connect(remote_ip,
                      username=config.ssh_admin_username, password=config.ssh_admin_password)
         conn.get_transport().set_keepalive(5)
         agent_transport = conn.get_transport()
         tunnel_thread = ForwardTunnelThread(
             local_port=local_port,
-            remote_host=execution.emulation_env_config.elk_config.container.get_ips()[0],
-            remote_port=execution.emulation_env_config.elk_config.kibana_port, transport=agent_transport,
+            remote_host=remote_ip,
+            remote_port=remote_port, transport=agent_transport,
             tunnels_dict=tunnels_dict)
         tunnel_thread.start()
         tunnel_thread_dict = {}
         tunnel_thread_dict[api_constants.MGMT_WEBAPP.THREAD_PROPERTY] = tunnel_thread
         tunnel_thread_dict[api_constants.MGMT_WEBAPP.PORT_PROPERTY] = local_port
-        tunnels_dict[execution.emulation_env_config.elk_config.container.get_ips()[0]] = tunnel_thread_dict
+        tunnels_dict[remote_ip] = tunnel_thread_dict
